"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExponentialCurve = exports.fromCurve = exports.amountAsNum = exports.asDecimal = exports.supplyAsNum = void 0;
const spl_token_1 = require("@solana/spl-token");
function supplyAsNum(mint) {
    return amountAsNum(mint.supply, mint);
}
exports.supplyAsNum = supplyAsNum;
function asDecimal(percent) {
    return percent / 4294967295; // uint32 max value
}
exports.asDecimal = asDecimal;
function amountAsNum(amount, mint) {
    const decimals = new spl_token_1.u64(Math.pow(10, mint.decimals).toString());
    const decimal = amount.mod(decimals).toNumber() / decimals.toNumber();
    return amount.div(decimals).toNumber() + decimal;
}
exports.amountAsNum = amountAsNum;
function fromCurve(curve, baseStorage, baseMint, targetMint) {
    switch (Object.keys(curve.definition)[0]) {
        case "timeV0":
            const curv = curve.definition.timeV0.curves[0].curve.exponentialCurveV0;
            return new ExponentialCurve(curv, baseStorage, baseMint, targetMint);
    }
    throw new Error("Curve not found");
}
exports.fromCurve = fromCurve;
class ExponentialCurve {
    constructor(curve, baseStorage, baseMint, targetMint) {
        this.c = curve.c.toNumber() / 1000000000000;
        this.b = curve.b.toNumber() / 1000000000000;
        this.k = curve.pow / curve.frac;
        this.pow = curve.pow;
        this.frac = curve.frac;
        this.baseStorage = baseStorage;
        this.baseMint = baseMint;
        this.targetMint = targetMint;
    }
    buyWithBaseRootEstimates(baseAmountNum, baseRoyaltiesPercent) {
        const S = supplyAsNum(this.targetMint);
        const R = amountAsNum(this.baseStorage.amount, this.baseMint);
        const dR = baseAmountNum * (1 - asDecimal(baseRoyaltiesPercent));
        if (R == 0 || S == 0) {
            return [Math.pow((dR * (1 + this.k)) / this.c, 1 / (1 + this.k)), 1];
        }
        else {
            /*
              dS = -S + ((S^(1 + k) (R + dR))/R)^(1/(1 + k))
            */
            return [
                Math.pow(S, 1 + this.k),
                Math.pow((Math.pow(S, 1 + this.k) * (R + dR)) / R, 1 / (1 + this.k)),
            ];
        }
    }
    buyTargetAmountRootEstimates(targetAmountNum, targetRoyaltiesPercent) {
        const S = supplyAsNum(this.targetMint);
        const dS = targetAmountNum * (1 / (1 - asDecimal(targetRoyaltiesPercent)));
        const R = amountAsNum(this.baseStorage.amount, this.baseMint);
        if (R == 0 || S == 0) {
            if (this.b == 0 && this.c != 0) {
                return [Math.pow(targetAmountNum, 1 + this.k), 1];
            }
            return [0, 0];
        }
        else {
            /*
              (R / S^(1 + k)) ((S + dS)^(1 + k) - S^(1 + k))
            */
            return [Math.pow(S, 1 + this.k), Math.pow(S + dS, 1 + this.k)];
        }
    }
    current() {
        return this.changeInTargetAmount(1, 0, 0);
    }
    locked() {
        return amountAsNum(this.baseStorage.amount, this.baseMint);
    }
    changeInTargetAmount(targetAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent) {
        // Calculate with the actual target amount they will need to get the target amount after royalties
        const dS = targetAmountNum * (1 / (1 - asDecimal(targetRoyaltiesPercent)));
        if (this.baseStorage.amount.toNumber() == 0 ||
            this.targetMint.supply.toNumber() == 0) {
            // b dS + (c dS^(1 + k))/(1 + k)
            return ((this.b * dS + (this.c * Math.pow(dS, 1 + this.k)) / (1 + this.k)) *
                (1 / (1 - asDecimal(baseRoyaltiesPercent))));
        }
        else {
            /*
              (R / S^(1 + k)) ((S + dS)(S + dS)^k - S^(1 + k))
            */
            const R = amountAsNum(this.baseStorage.amount, this.baseMint);
            const S = supplyAsNum(this.targetMint);
            return (((R / Math.pow(S, 1 + this.k)) *
                ((S + dS) * Math.pow(S + dS, this.k) - Math.pow(S, 1 + this.k))) /
                (1 - asDecimal(baseRoyaltiesPercent)));
        }
    }
    sellTargetAmount(targetAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent) {
        return -this.changeInTargetAmount(-targetAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent);
    }
    buyTargetAmount(targetAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent) {
        return this.changeInTargetAmount(targetAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent);
    }
    buyWithBaseAmount(baseAmountNum, baseRoyaltiesPercent, targetRoyaltiesPercent) {
        const dR = baseAmountNum * (1 - asDecimal(baseRoyaltiesPercent));
        if (this.baseStorage.amount.toNumber() == 0 ||
            this.targetMint.supply.toNumber() == 0) {
            if (this.b == 0) {
                /*
                 * -S + (((1 + k) dR)/c)^(1/(1 + k))
                 */
                return ((Math.pow(((1 + this.k) * dR) / this.c, 1 / (1 + this.k)) -
                    supplyAsNum(this.targetMint)) *
                    (1 - asDecimal(targetRoyaltiesPercent)));
            }
            else if (this.k == 0) {
                return dR / this.b;
            }
            throw new Error("Cannot convert base amount to target amount when both b and k are defined on an exponential curve. The math is too hard");
        }
        else {
            const R = amountAsNum(this.baseStorage.amount, this.baseMint);
            const S = supplyAsNum(this.targetMint);
            /*
             * dS = -S + ((S^(1 + k) (R + dR))/R)^(1/(1 + k))
             */
            return ((-S +
                Math.pow((Math.pow(S, 1 + this.k) * (R + dR)) / R, 1 / (1 + this.k))) *
                (1 - asDecimal(targetRoyaltiesPercent)));
        }
    }
}
exports.ExponentialCurve = ExponentialCurve;
//# sourceMappingURL=curves.js.map