import * as anchor from "@project-serum/anchor";
import { Program, Provider } from "@project-serum/anchor";
import { AccountInfo, MintInfo } from "@solana/spl-token";
import { Keypair, PublicKey, Signer, TransactionInstruction } from "@solana/web3.js";
import { InstructionResult, TypedAccountParser } from "@strata-foundation/spl-utils";
import BN from "bn.js";
import { IPricingCurve } from "./curves";
import { CurveV0, ProgramStateV0, SplTokenBondingIDL, TokenBondingV0 } from "./generated/spl-token-bonding";
export * from "./curves";
export * from "./generated/spl-token-bonding";
/**
 * The curve config required by the smart contract is unwieldy, implementors of `CurveConfig` wrap the interface
 */
interface ICurveConfig {
    toRawConfig(): CurveV0;
}
interface IPrimitiveCurve {
    toRawPrimitiveConfig(): any;
}
/**
 * Convert a number to a 12 decimal fixed precision u128
 *
 * @param num Number to convert to a 12 decimal fixed precision BN
 * @returns
 */
export declare function toU128(num: number | BN): BN;
/**
 * Curve configuration for c(S^(pow/frac)) + b
 */
export declare class ExponentialCurveConfig implements ICurveConfig, IPrimitiveCurve {
    c: BN;
    b: BN;
    pow: number;
    frac: number;
    constructor({ c, b, pow, frac, }: {
        c?: number | BN;
        b?: number | BN;
        pow?: number;
        frac?: number;
    });
    toRawPrimitiveConfig(): any;
    toRawConfig(): CurveV0;
}
/**
 * Curve configuration that allows the curve to change parameters at discrete time offsets from the go live date
 */
export declare class TimeCurveConfig implements ICurveConfig {
    curves: {
        curve: IPrimitiveCurve;
        offset: BN;
    }[];
    addCurve(timeOffset: number, curve: IPrimitiveCurve): TimeCurveConfig;
    toRawConfig(): CurveV0;
}
export interface IInitializeCurveArgs {
    /** The configuration for the shape of curve */
    config: ICurveConfig;
    /** The payer to create this curve, defaults to provider.wallet */
    payer?: PublicKey;
}
export interface ICreateTokenBondingArgs {
    /** The payer to create this token bonding, defaults to provider.wallet */
    payer?: PublicKey;
    /** The shape of the bonding curve. Must be created using {@link SplTokenBonding.initializeCurve} */
    curve: PublicKey;
    /** The base mint that the `targetMint` will be priced in terms of. `baseMint` tokens will fill the bonding curve reserves */
    baseMint: PublicKey;
    /**
     * The mint this bonding curve will create on `buy`. If not provided, specify `targetMintDecimals` and it will create one for you
     *
     * It can be useful to pass the mint in if you're creating a bonding curve for an existing mint. Keep in mind,
     * the authority on this mint will need to be set to:
     * ```js
     * PublicKey.findProgramAddress(
        [
          Buffer.from("target-authority", "utf-8"),
          targetMint!.toBuffer()
        ],
        this.programId
      )
     * ```
     */
    targetMint?: PublicKey;
    /**
     * **Default:** New generated keypair
     *
     * Pass in the keypair to use for the mint. Useful if you want a vanity keypair
     */
    targetMintKeypair?: anchor.web3.Keypair;
    /** If `targetMint` is not defined, will create a mint with this number of decimals */
    targetMintDecimals?: number;
    /**
     * Account to store royalties in terms of `baseMint` tokens when the {@link SplTokenBonding.buy} command is issued
     *
     * If not provided, will create an Associated Token Account with `buyBaseRoyaltiesOwner`
     */
    buyBaseRoyalties?: PublicKey;
    /** Only required when `buyBaseRoyalties` is undefined. The owner of the `buyBaseRoyalties` account. **Default:** `provider.wallet` */
    buyBaseRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `targetMint` tokens when the {@link SplTokenBonding.buy} command is issued
     *
     * If not provided, will create an Associated Token Account with `buyTargetRoyaltiesOwner`
     */
    buyTargetRoyalties?: PublicKey;
    /** Only required when `buyTargetRoyalties` is undefined. The owner of the `buyTargetRoyalties` account. **Default:** `provider.wallet` */
    buyTargetRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `baseMint` tokens when the {@link SplTokenBonding.sell} command is issued
     *
     * If not provided, will create an Associated Token Account with `sellBaseRoyaltiesOwner`
     */
    sellBaseRoyalties?: PublicKey;
    /** Only required when `sellBaseRoyalties` is undefined. The owner of the `sellBaseRoyalties` account. **Default:** `provider.wallet` */
    sellBaseRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `targetMint` tokens when the {@link SplTokenBonding.sell} command is issued
     *
     * If not provided, will create an Associated Token Account with `sellTargetRoyaltiesOwner`
     */
    sellTargetRoyalties?: PublicKey;
    /** Only required when `sellTargetRoyalties` is undefined. The owner of the `sellTargetRoyalties` account. **Default:** `provider.wallet` */
    sellTargetRoyaltiesOwner?: PublicKey;
    authority?: PublicKey;
    /**
     * The reserves of the bonding curve. When {@link SplTokenBonding.buy} is called, `baseMint` tokens are stored here.
     * When {@link SplTokenBonding.sell} is called, `baseMint` tokens are returned to the callee from this account
     *
     * Optionally, this account can have an authority _not_ owned by the spl-token-bonding program. In this case, a bonding curve
     * is created with {@link SplTokenBonding.sell} disabled. This allows the bonding curve contract to be used like a
     * marketplace to sell a new token
     *
     * **Default:** creates this account for you, owned by the token bonding program
     */
    baseStorage?: PublicKey;
    /** Number from 0 to 100 */
    buyBaseRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    buyTargetRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    sellBaseRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    sellTargetRoyaltyPercentage: number;
    /** Maximum `targetMint` tokens this bonding curve will mint before disabling {@link SplTokenBonding.buy}. **Default:** infinite */
    mintCap?: BN;
    /** Maximum `targetMint` tokens that can be purchased in a single call to {@link SplTokenBonding.buy}. Useful for limiting volume. **Default:** 0 */
    purchaseCap?: BN;
    /** The date this bonding curve will go live. Before this date, {@link SplTokenBonding.buy} and {@link SplTokenBonding.sell} are disabled. **Default:** 1 second ago */
    goLiveDate?: Date;
    /** The date this bonding curve will shut down. After this date, {@link SplTokenBonding.buy} and {@link SplTokenBonding.sell} are disabled. **Default:** null */
    freezeBuyDate?: Date;
    /** Should this bonding curve be frozen initially? It can be unfrozen using {@link SplTokenBonding.updateTokenBonding}. **Default:** false */
    buyFrozen?: boolean;
    /**
     * Multiple bonding curves can exist for a given target mint.
     * 0 is reserved for the one where the program owns mint authority and can mint new tokens. All other curves may exist as
     * markeplace curves
     */
    index?: number;
}
export interface IUpdateTokenBondingArgs {
    /** The bonding curve to update */
    tokenBonding: PublicKey;
    /** Number from 0 to 100. **Default:** current */
    buyBaseRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    buyTargetRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    sellBaseRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    sellTargetRoyaltyPercentage?: number;
    /** A new account to store royalties. **Default:** current */
    buyBaseRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    buyTargetRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    sellBaseRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    sellTargetRoyalties?: PublicKey;
    authority?: PublicKey | null;
    /** Should this bonding curve be frozen, disabling buy and sell? It can be unfrozen using {@link SplTokenBonding.updateTokenBonding}. **Default:** current */
    buyFrozen?: boolean;
}
export interface IBuyArgs {
    tokenBonding: PublicKey;
    /** The payer to run this transaction, defaults to provider.wallet */
    payer?: PublicKey;
    source?: PublicKey;
    destination?: PublicKey;
    sourceAuthority?: PublicKey;
    desiredTargetAmount?: BN | number;
    baseAmount?: BN | number;
    slippage: number;
}
export interface ISellArgs {
    tokenBonding: PublicKey;
    /** The payer to run this transaction, defaults to provider.wallet */
    payer?: PublicKey;
    source?: PublicKey /** `targetMint` source account to sell from. **Default:** ATA of sourceAuthority */;
    destination?: PublicKey /** `baseMint` destination for tokens from the reserve. **Default:** ATA of wallet */;
    sourceAuthority?: PublicKey /** **Default:** wallet */;
    targetAmount: BN | number /** The amount of `targetMint` tokens to sell. */;
    slippage: number;
}
/**
 * Unified token bonding interface wrapping the raw TokenBondingV0
 */
export interface ITokenBonding extends TokenBondingV0 {
    publicKey: PublicKey;
}
/**
 * Unified curve interface wrapping the raw CurveV0
 */
export interface ICurve extends CurveV0 {
    publicKey: PublicKey;
}
export declare class SplTokenBonding {
    program: Program<SplTokenBondingIDL>;
    provider: Provider;
    state: ProgramStateV0 | undefined;
    static ID: anchor.web3.PublicKey;
    static init(provider: Provider, splTokenBondingProgramId?: PublicKey): Promise<SplTokenBonding>;
    constructor(provider: Provider, program: Program<SplTokenBondingIDL>);
    curveDecoder: TypedAccountParser<ICurve>;
    tokenBondingDecoder: TypedAccountParser<ITokenBonding>;
    get programId(): anchor.web3.PublicKey;
    get rpc(): anchor.RpcNamespace<SplTokenBondingIDL, {
        name: "initializeSolStorageV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "state";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeSolStorageV0Args";
            };
        }];
    } | {
        name: "buyWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyWrappedSolV0Args";
            };
        }];
    } | {
        name: "sellWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "owner";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellWrappedSolV0Args";
            };
        }];
    } | {
        name: "createCurveV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: true;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "CreateCurveV0Args";
            };
        }];
    } | {
        name: "initializeTokenBondingV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeTokenBondingV0Args";
            };
        }];
    } | {
        name: "closeTokenBondingV0";
        accounts: [{
            name: "refund";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [];
    } | {
        name: "updateTokenBondingV0";
        accounts: [{
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint"; /**
             * Runs {@link `createTokenBondingInstructions`}
             *
             * @param args
             * @returns
             */
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "UpdateTokenBondingV0Args";
            };
        }]; /**
         * Update a bonding curve.
         *
         * @param param0
         * @returns
         */
    } | {
        name: "buyV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyV0Args";
            };
        }];
    } | {
        name: "sellV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellV0Args";
            };
        }];
    }>;
    get instruction(): anchor.InstructionNamespace<SplTokenBondingIDL, {
        name: "initializeSolStorageV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "state";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeSolStorageV0Args";
            };
        }];
    } | {
        name: "buyWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "mintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyWrappedSolV0Args";
            };
        }];
    } | {
        name: "sellWrappedSolV0";
        accounts: [{
            name: "state";
            isMut: false;
            isSigner: false;
        }, {
            name: "wrappedSolMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "solStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "owner";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellWrappedSolV0Args";
            };
        }];
    } | {
        name: "createCurveV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: true;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "CreateCurveV0Args";
            };
        }];
    } | {
        name: "initializeTokenBondingV0";
        accounts: [{
            name: "payer";
            isMut: true;
            isSigner: true;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "systemProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "rent";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "InitializeTokenBondingV0Args";
            };
        }];
    } | {
        name: "closeTokenBondingV0";
        accounts: [{
            name: "refund";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }];
        args: [];
    } | {
        name: "updateTokenBondingV0";
        accounts: [{
            name: "tokenBonding";
            isMut: true;
            isSigner: false;
        }, {
            name: "authority";
            isMut: false;
            isSigner: true;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint"; /**
             * Runs {@link `createTokenBondingInstructions`}
             *
             * @param args
             * @returns
             */
            isMut: false;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: false;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "UpdateTokenBondingV0Args";
            };
        }]; /**
         * Update a bonding curve.
         *
         * @param param0
         * @returns
         */
    } | {
        name: "buyV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "targetMintAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "buyTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "BuyV0Args";
            };
        }];
    } | {
        name: "sellV0";
        accounts: [{
            name: "tokenBonding";
            isMut: false;
            isSigner: false;
        }, {
            name: "curve";
            isMut: false;
            isSigner: false;
        }, {
            name: "baseMint";
            isMut: false;
            isSigner: false;
        }, {
            name: "targetMint";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorage";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellBaseRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "sellTargetRoyalties";
            isMut: true;
            isSigner: false;
        }, {
            name: "baseStorageAuthority";
            isMut: false;
            isSigner: false;
        }, {
            name: "source";
            isMut: true;
            isSigner: false;
        }, {
            name: "sourceAuthority";
            isMut: false;
            isSigner: true;
        }, {
            name: "destination";
            isMut: true;
            isSigner: false;
        }, {
            name: "tokenProgram";
            isMut: false;
            isSigner: false;
        }, {
            name: "clock";
            isMut: false;
            isSigner: false;
        }];
        args: [{
            name: "args";
            type: {
                defined: "SellV0Args";
            };
        }];
    }>;
    get wallet(): import("@project-serum/anchor/dist/cjs/provider").Wallet;
    get account(): anchor.AccountNamespace<SplTokenBondingIDL>;
    get errors(): Map<number, string>;
    sendInstructions(instructions: TransactionInstruction[], signers: Signer[], payer?: PublicKey): Promise<string>;
    /**
     * This is an admin function run once to initialize the smart contract.
     *
     * @returns Instructions needed to create sol storage
     */
    initializeSolStorageInstructions(): Promise<InstructionResult<null>>;
    /**
     * Admin command run once to initialize the smart contract
     */
    initializeSolStorage(): Promise<void>;
    /**
     * Create a curve shape for use in a TokenBonding instance
     *
     * @param param0
     * @returns
     */
    initializeCurveInstructions({ payer, config: curveConfig, }: IInitializeCurveArgs): Promise<InstructionResult<{
        curve: PublicKey;
    }>>;
    /**
     * See {@link initializeCurve}
     * @param args
     * @returns
     */
    initializeCurve(args: IInitializeCurveArgs): Promise<PublicKey>;
    /**
     * Get the PDA key of a TokenBonding given the target mint and index
     *
     * `index` = 0 is the default bonding curve that can mint `targetMint`. All other curves are curves that allow burning of `targetMint` for some different base.
     *
     * @param targetMint
     * @param index
     * @returns
     */
    tokenBondingKey(targetMint: PublicKey, index: number): Promise<[PublicKey, number]>;
    /**
     * Get the PDA key of the account that should be the authority on the base storage (reserve) account of a bonding curve that doesn't have sell frozen.
     * @param tokenBonding
     * @returns
     */
    baseStorageAuthorityKey(tokenBonding: PublicKey): Promise<[PublicKey, number]>;
    /**
     * Create a bonding curve
     *
     * @param param0
     * @returns
     */
    createTokenBondingInstructions({ authority, payer, curve, baseMint, targetMint, baseStorage, buyBaseRoyalties, buyBaseRoyaltiesOwner, buyTargetRoyalties, buyTargetRoyaltiesOwner, sellBaseRoyalties, sellBaseRoyaltiesOwner, sellTargetRoyalties, sellTargetRoyaltiesOwner, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, mintCap, purchaseCap, goLiveDate, // 1 secs ago
    freezeBuyDate, targetMintDecimals, targetMintKeypair, buyFrozen, index, }: ICreateTokenBondingArgs): Promise<InstructionResult<{
        tokenBonding: PublicKey;
        targetMint: PublicKey;
        buyBaseRoyalties: PublicKey;
        buyTargetRoyalties: PublicKey;
        sellBaseRoyalties: PublicKey;
        sellTargetRoyalties: PublicKey;
        baseStorage: PublicKey;
    }>>;
    /**
     * General utility function to check if an account exists
     * @param account
     * @returns
     */
    accountExists(account: anchor.web3.PublicKey): Promise<boolean>;
    /**
     * Runs {@link `createTokenBondingInstructions`}
     *
     * @param args
     * @returns
     */
    createTokenBonding(args: ICreateTokenBondingArgs): Promise<PublicKey>;
    /**
     * Update a bonding curve.
     *
     * @param param0
     * @returns
     */
    updateTokenBondingInstructions({ tokenBonding, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, buyBaseRoyalties, buyTargetRoyalties, sellBaseRoyalties, sellTargetRoyalties, authority, buyFrozen, }: IUpdateTokenBondingArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link updateTokenBonding}
     * @param args
     */
    updateTokenBonding(args: IUpdateTokenBondingArgs): Promise<void>;
    /**
     * Create a temporary account with `amount` twSOL, the token bonding wrapped sol mint.
     *
     * @param param0
     * @returns
     */
    createTemporaryWSolAccount({ payer, owner, amount, }: {
        owner: PublicKey;
        payer: PublicKey;
        amount: number;
    }): Promise<{
        signer: Keypair;
        firstInstructions: TransactionInstruction[];
        lastInstructions: TransactionInstruction[];
    }>;
    /**
     * Issue a command to buy `targetMint` tokens with `baseMint` tokens.
     *
     * @param param0
     * @returns
     */
    buyInstructions({ tokenBonding, source, sourceAuthority, destination, desiredTargetAmount, baseAmount, slippage, payer, }: IBuyArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link buy}
     * @param args
     */
    buy(args: IBuyArgs): Promise<void>;
    getState(): Promise<ProgramStateV0 | null>;
    /**
     * Instructions to burn `targetMint` tokens in exchange for `baseMint` tokens
     *
     * @param param0
     * @returns
     */
    sellInstructions({ tokenBonding, source, sourceAuthority, destination, targetAmount, slippage, payer, }: ISellArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link sell}
     * @param args
     */
    sell(args: ISellArgs): Promise<void>;
    /**
     * Get a class capable of displaying pricing information or this token bonding at its current reserve and supply
     *
     * @param tokenBonding
     * @returns
     */
    getPricing(tokenBonding: PublicKey): Promise<IPricingCurve>;
    /**
     * Given some reserves and supply, get a pricing model for a curve at `key`.
     *
     * @param key
     * @param baseStorage
     * @param baseMint
     * @param targetMint
     * @returns
     */
    getCurve(key: PublicKey, baseStorage: AccountInfo, baseMint: MintInfo, targetMint: MintInfo): Promise<IPricingCurve>;
}
//# sourceMappingURL=index.d.ts.map